==> ifrit_3.2.0.orig.tar.gz: ifrit-3.2.0/core/iparticlefileloader.h <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/

//
//  Abstract DataLoader for reading various particle files
//
#ifndef IPARTICLEFILELOADER_H
#define IPARTICLEFILELOADER_H


#include "ifileloader.h"


#include "iparticledownsampleiterator.h"

class iParticleDensityEstimator;
class iParticleHelper;

class vtkCellArray;
class vtkDataArray;
class vtkDoubleArray;
class vtkFloatArray;
class vtkPoints;
class vtkPolyData;


class iParticleFileLoader : public iFileLoader
{

	friend class iBasicParticlesFileLoader;
	friend class iParticleHelper;

public:

	vtkTypeMacro(iParticleFileLoader,iFileLoader);

	void SetDownsampleMode(int v);
	int GetDownsampleMode() const;

	void SetTypeIncluded(int type, bool s);
	bool GetTypeIncluded(int type) const;

	void SetDownsampleFactor(int type, int value);
	int GetDownsampleFactor(int type) const;

	void SetDensityAttribute(int type, int attr);
	int GetDensityAttribute(int type) const;

	void SetOrderIsAttribute(int type, bool s);
	bool GetOrderIsAttribute(int type) const;

	void SetNumNeighbors(int num);
	int GetNumNeighbors() const;

protected:

	iParticleFileLoader(iDataReaderExtension *ext, int priority = 300);
	virtual ~iParticleFileLoader();

	struct ParticleStreamExtras
	{
		int DensityAttribute;
		bool OrderIsAttribute;

		ParticleStreamExtras();
	};

	struct ParticleStream : public Stream
	{
		bool Included, InFile;
		int DownsampleFactor;
		int NumAttributesInFile, NumAttributesInData;
		long NumTotal;
		vtkIdType NumSelected;
		vtkPolyData *Data;
		vtkPoints *Points;
		vtkCellArray *Verts;
		vtkFloatArray *Scalars;
		ParticleStreamExtras Extras;

		ParticleStream();
	};
	virtual Stream* CreateNewStream() const;
	inline ParticleStream* GetStream(int i) const { return static_cast<ParticleStream*>(iFileLoader::GetStream(i)); } // no bound checking for speed

	virtual void FinalizeBody();
	virtual void ShiftDataBody(vtkDataSet *data, double dx[3]);

	virtual void AddExtras(ParticleStream *stream);

	//
	//  helper functions
	//
	void FinalizePolyData(vtkPolyData *data);
	void ShiftPolyData(vtkPolyData *data, int d, double dx);

	bool ReadPositions(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, float *scale, float *offset);
	bool ReadPositions(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, double *scale, double *offset);
	bool ReadAttributes(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, float *scale = 0);
	bool ReadIntAttributes(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration);

	void AutoScalePositions();

	void ConfigureStreams(int *ntot, int *natt, bool paf = true);
	void ConfigureStreams(long *ntot, int *natt, bool paf = true);
	void ReleaseStreams();
	void SetErrorMessage(const iString &message);

	iParticleDownsampleIterator mIterator; // value, not the pointer, for efficiency (to remove one level of indirection in loops)
	iParticleDensityEstimator *mDensityEstimator;

private:

	iParticleFileLoader(iDataReader *r, int priority = 300);

	void Define();

	//
	//  Members
	//
	bool mHaveNormals;
	iParticleHelper *mHelper;
};


inline bool iParticleFileLoader::GetTypeIncluded(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Included;
	}
	else return false;
}


inline int iParticleFileLoader::GetDownsampleFactor(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->DownsampleFactor;
	}
	else return 1;
}


inline int iParticleFileLoader::GetDensityAttribute(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Extras.DensityAttribute;
	}
	else return 1;
}


inline bool iParticleFileLoader::GetOrderIsAttribute(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Extras.OrderIsAttribute;
	}
	else return false;
}

#endif

==> ifrit_3.3.2.orig.tar.gz: ifrit-3.3.2/core/iparticlefileloader.h <==
/*=========================================================================

  Program:   Ionization FRont Interactive Tool (IFRIT)
  Language:  C++


Copyright (c) 2002-2006 Nick Gnedin 
All rights reserved.

This file may be distributed and/or modified under the terms of the
GNU General Public License version 2 as published by the Free Software
Foundation and appearing in the file LICENSE.GPL included in the
packaging of this file.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/

//
//  Abstract DataLoader for reading various particle files
//
#ifndef IPARTICLEFILELOADER_H
#define IPARTICLEFILELOADER_H


#include "ifileloader.h"


#include "iparticledownsampleiterator.h"

class iParticleDensityEstimator;
class iParticleHelper;

class vtkCellArray;
class vtkDataArray;
class vtkDoubleArray;
class vtkFloatArray;
class vtkPoints;
class vtkPolyData;


class iParticleFileLoader : public iFileLoader
{

	friend class iBasicParticlesFileLoader;
	friend class iParticleHelper;

public:

	vtkTypeMacro(iParticleFileLoader,iFileLoader);

	void SetDownsampleMode(int v);
	int GetDownsampleMode() const;

	void SetTypeIncluded(int type, bool s);
	bool GetTypeIncluded(int type) const;

	void SetDownsampleFactor(int type, int value);
	int GetDownsampleFactor(int type) const;

	void SetDensityAttribute(int type, int attr);
	int GetDensityAttribute(int type) const;

	void SetOrderIsAttribute(int type, bool s);
	bool GetOrderIsAttribute(int type) const;

	void SetNumNeighbors(int num);
	int GetNumNeighbors() const;

	inline bool IsUsingExtras() const { return mUseExtras; }

protected:

	iParticleFileLoader(iDataReaderExtension *ext, int priority = 300, bool useExtras = true);
	virtual ~iParticleFileLoader();

	struct ParticleStreamExtras
	{
		int DensityAttribute;
		bool OrderIsAttribute;

		ParticleStreamExtras();
	};

	struct ParticleStream : public Stream
	{
		bool Included, InFile;
		int DownsampleFactor;
		int NumAttributesInFile, NumAttributesInData;
		vtkIdType NumTotal;
		vtkIdType NumSelected;
		vtkPolyData *Data;
		vtkPoints *Points;
		vtkCellArray *Verts;
		vtkFloatArray *Scalars;
		ParticleStreamExtras Extras;

		ParticleStream();
	};
	virtual Stream* CreateNewStream() const;
	inline ParticleStream* GetStream(int i) const { return static_cast<ParticleStream*>(iFileLoader::GetStream(i)); } // no bound checking for speed

	virtual void FinalizeBody();
	virtual void ShiftDataBody(vtkDataSet *data, double dx[3]);

	virtual void AddExtras(ParticleStream *stream);

	//
	//  helper functions
	//
	void FinalizePolyData(vtkPolyData *data);
	void ShiftPolyData(vtkPolyData *data, int d, double dx);

	bool ReadPositions(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, float *scale, float *offset);
	bool ReadPositions(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, double *scale, double *offset);
	bool ReadAttributes(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration, float *scale = 0);
	bool ReadIntAttributes(iFile &F, int inNum, int outOffset, float updateStart, float updateDuration);

	void AutoScalePositions();

	void ConfigureStreams(vtkIdType *ntot, int *natt, bool paf = true);
	void ReleaseStreams();
	void SetErrorMessage(const iString &message);

	iParticleDownsampleIterator mIterator; // value, not the pointer, for efficiency (to remove one level of indirection in loops)
	iParticleDensityEstimator *mDensityEstimator;

private:

	iParticleFileLoader(iDataReader *r, int priority = 300, bool useExtras = true);

	void Define();

	//
	//  Members
	//
	const bool mUseExtras;
	bool mHaveNormals;
	iParticleHelper *mHelper;
};


inline bool iParticleFileLoader::GetTypeIncluded(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Included;
	}
	else return false;
}


inline int iParticleFileLoader::GetDownsampleFactor(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->DownsampleFactor;
	}
	else return 1;
}


inline int iParticleFileLoader::GetDensityAttribute(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Extras.DensityAttribute;
	}
	else return 1;
}


inline bool iParticleFileLoader::GetOrderIsAttribute(int type) const
{
	if(type>=0 && type<this->NumStreams())
	{
		return this->GetStream(type)->Extras.OrderIsAttribute;
	}
	else return false;
}

#endif
